@@ -242,6 +242,8 @@ class DefaultLabelDialog(QtWidgets.QDialog):
 
 
 class LabelDialogExt(DefaultLabelDialog):
+    """ 双击Label，打开的查看、编辑label框
+    """
     def __init__(
             self,
             text="Enter object label",
@@ -409,8 +411,8 @@ class LabelDialogExt(DefaultLabelDialog):
         """ 增加布局项 """
         # 1 初始化
         self.initLayout()
-        labelcfg = mainwin.labelcfg
-        showtext, hashtext, labelattr = labelcfg.parse_shape(shape)
+        xllabel = mainwin.xllabel
+        showtext, hashtext, labelattr = xllabel.parse_shape(shape)
         if not labelattr:
             return
         layout = self.layout()
@@ -418,7 +420,7 @@ class LabelDialogExt(DefaultLabelDialog):
         # 2 处理labelattr的场景
         # 2.1 原有标准控件调整
         self.labelList.hide()
-        self.edit.setEnabled(labelcfg.cfg['editable'])
+        self.edit.setEnabled(xllabel.cfg['editable'])
 
         # 2.2 封装添加每行控件的功能
         n_row = 1
@@ -428,7 +430,7 @@ class LabelDialogExt(DefaultLabelDialog):
 
             def edit_label(k, x):
                 """ 回调函数，控件修改值后，实时更新label """
-                self.edit.setText(labelcfg.set_label_attr(self.edit.text(), k, x))
+                self.edit.setText(xllabel.set_label_attr(self.edit.text(), k, x))
                 self.setWindowModified(True)
 
             if k == 'label' and v and v['items'] is None and self.labelList.count():
@@ -457,12 +459,12 @@ class LabelDialogExt(DefaultLabelDialog):
         # 2.3.1 控件分三组：明确不显示，明确要显示，未在设定清单中
         # labelattr中的所有字段都应该要展示，但要优先展示配置项里指定的，再展示其他次要、衍生字段
         keys_group = [[], [], []]
-        for x in labelcfg.cfg['attrs']:
+        for x in xllabel.cfg['attrs']:
             k, s = x['key'], x['show']
             if k in labelattr:
                 keys_group[s].append(k)
         for k in labelattr.keys():
-            if k not in labelcfg.keys:
+            if k not in xllabel.keys:
                 keys_group[2].append(k)
         # 2.3.2
         for g in [1, 2, 0]:  # 展示每组的顺序
@@ -470,7 +472,7 @@ class LabelDialogExt(DefaultLabelDialog):
                 layout.insertRow(n_row, QHLine())  # 加分割线
                 n_row += 1
                 for k in keys_group[g]:
-                    add_row(k, labelcfg.get(k, None))
+                    add_row(k, xllabel.get(k, None))
         layout.insertRow(n_row, QHLine())
 
     def popUp2(self, shape, mainwin=None):
